@@ -23,14 +23,24 @@
 #include <qb/qbarray.h>
 #include <qb/qbutil.h>
 
-#define MAX_ELEMENTS_PER_BIN 16
-#define MAX_BINS 4096
+/* The highest ARRAY_INDEX_BITS_BINS bits of the array index are the
+ * number of the bin containing the indicated element, while the
+ * remaining ARRAY_INDEX_BITS_ELEMS_PER_BIN bits give its index inside
+ * the bin.  The full array index is ARRAY_INDEX_BITS_MAX bits long.
+ */
 
-#define BIN_NUM_GET(_idx_) (_idx_ >> 4)
-#define ELEM_NUM_GET(_idx_) (_idx_ & 0x0F)
+#define ARRAY_INDEX_BITS_ELEMS_PER_BIN 4
+#define ARRAY_INDEX_BITS_BINS \
+	(QB_ARRAY_MAX_INDEX_BITS - ARRAY_INDEX_BITS_ELEMS_PER_BIN)
+
+#define MAX_ELEMENTS_PER_BIN (1 << ARRAY_INDEX_BITS_ELEMS_PER_BIN)
+#define MAX_BINS (1 << ARRAY_INDEX_BITS_BINS)
+
+#define BIN_NUM_GET(_idx_) (_idx_ >> ARRAY_INDEX_BITS_ELEMS_PER_BIN)
+#define ELEM_NUM_GET(_idx_) (_idx_ & (MAX_ELEMENTS_PER_BIN - 1))
 
 struct qb_array {
-	void **bin;
+	void **bin;  /* array of void* pointers to element_size big elements */
 	size_t max_elements;
 	size_t element_size;
 	size_t num_bins;
@@ -46,9 +56,9 @@ qb_array_create(size_t max_elements, size_t element_size)
 }
 
 static int32_t
-_grow_bin_array(struct qb_array * a, int32_t new_bin_size)
+_grow_bin_array(struct qb_array * a, size_t new_bin_size)
 {
-	int32_t b;
+	size_t b;
 
 	a->bin = realloc(a->bin, sizeof(void*) * new_bin_size);
 	if (a->bin == NULL) {
@@ -67,18 +77,18 @@ qb_array_create_2(size_t max_elements, size_t element_size,
 		  size_t autogrow_elements)
 {
 	struct qb_array *a = NULL;
-	int32_t b;
+	size_t b;
 
-	if (max_elements > (MAX_ELEMENTS_PER_BIN * MAX_BINS)) {
-		errno = -EINVAL;
+	if (max_elements > QB_ARRAY_MAX_ELEMENTS) {
+		errno = EINVAL;
 		return NULL;
 	}
 	if (element_size < 1) {
-		errno = -EINVAL;
+		errno = EINVAL;
 		return NULL;
 	}
 	if (autogrow_elements > MAX_ELEMENTS_PER_BIN) {
-		errno = -EINVAL;
+		errno = EINVAL;
 		return NULL;
 	}
 	a = calloc(1, sizeof(struct qb_array));
@@ -101,7 +111,7 @@ qb_array_create_2(size_t max_elements, size_t element_size,
 int32_t
 qb_array_index(struct qb_array * a, int32_t idx, void **element_out)
 {
-	int32_t b;
+	size_t b;
 	int32_t elem;
 	char *bin;
 	int32_t rc = 0;
@@ -112,7 +122,7 @@ qb_array_index(struct qb_array * a, int32_t idx, void **element_out)
 	if (idx < 0) {
 		return -ERANGE;
 	}
-	if (idx >= a->max_elements) {
+	if ((uint32_t) idx >= a->max_elements) {
 		if (a->autogrow_elements == 0) {
 			return -ERANGE;
 		} else {
@@ -122,7 +132,7 @@ qb_array_index(struct qb_array * a, int32_t idx, void **element_out)
 			}
 		}
 	}
-	b = BIN_NUM_GET(idx);
+	b = BIN_NUM_GET((uint32_t) idx);
 	assert(b < MAX_BINS);
 
 	if (b >= a->num_bins || a->bin[b] == NULL) {
@@ -139,6 +149,7 @@ qb_array_index(struct qb_array * a, int32_t idx, void **element_out)
 		if (a->bin[b] == NULL) {
 			a->bin[b] = calloc(MAX_ELEMENTS_PER_BIN, a->element_size);
 			if (a->bin[b] == NULL) {
+				rc = -errno;
 				goto unlock_error;
 			}
 			bin_alloced = QB_TRUE;
@@ -195,10 +206,10 @@ qb_array_elems_per_bin_get(struct qb_array * a)
 int32_t
 qb_array_grow(struct qb_array * a, size_t max_elements)
 {
-	int32_t b;
+	size_t b;
 	int32_t rc = 0;
 
-	if (a == NULL || max_elements > (MAX_ELEMENTS_PER_BIN * MAX_BINS)) {
+	if (a == NULL || max_elements > QB_ARRAY_MAX_ELEMENTS) {
 		return -EINVAL;
 	}
 	if (max_elements <= a->max_elements) {
@@ -219,7 +230,7 @@ qb_array_grow(struct qb_array * a, size_t max_elements)
 void
 qb_array_free(struct qb_array *a)
 {
-	int32_t i;
+	size_t i;
 	for (i = 0; i < a->num_bins; i++) {
 		free(a->bin[i]);
 	}
