@@ -28,9 +28,9 @@
 #include <qb/qbutil.h>
 #include "log_int.h"
 
-static int wthread_active = 0;
+static int wthread_active = QB_FALSE;
 
-static int wthread_should_exit = 0;
+static int wthread_should_exit = QB_FALSE;
 
 static qb_thread_lock_t *logt_wthread_lock = NULL;
 
@@ -48,9 +48,9 @@ static int logt_sched_param_queued = QB_FALSE;
 
 static int logt_sched_policy;
 
-#if defined(HAVE_PTHREAD_SETSCHEDPARAM) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM)
 static struct sched_param logt_sched_param;
-#endif /* HAVE_PTHREAD_SETSCHEDPARAM && HAVE_SCHED_GET_PRIORITY_MAX */
+#endif /* HAVE_PTHREAD_SETSCHEDPARAM */
 
 static pthread_t logt_thread_id = 0;
 
@@ -62,9 +62,7 @@ qb_logt_worker_thread(void *data)
 	int dropped = 0;
 	int res;
 
-	/*
-	 * Signal wthread_create that the initialization process may continue
-	 */
+	/* Signal qb_log_thread_start that the initialization may continue */
 	sem_post(&logt_thread_start);
 	for (;;) {
 retry_sem_wait:
@@ -72,9 +70,7 @@ retry_sem_wait:
 		if (res == -1 && errno == EINTR) {
 			goto retry_sem_wait;
 		} else if (res == -1) {
-			/*
-			 * This case shouldn't happen
-			 */
+			/* This case shouldn't happen */
 			pthread_exit(NULL);
 		}
 
@@ -90,19 +86,20 @@ retry_sem_wait:
 		}
 
 		rec =
-		    qb_list_entry(logt_print_finished_records.next,
+		    qb_list_first_entry(&logt_print_finished_records,
 				  struct qb_log_record, list);
 		qb_list_del(&rec->list);
 		logt_memory_used = logt_memory_used - strlen(rec->buffer) -
 		    sizeof(struct qb_log_record) - 1;
 		dropped = logt_dropped_messages;
 		logt_dropped_messages = 0;
-		(void)qb_thread_unlock(logt_wthread_lock);
 		if (dropped) {
 			printf("%d messages lost\n", dropped);
 		}
 
-		qb_log_thread_log_write(rec->cs, rec->timestamp, rec->buffer);
+		qb_log_thread_log_write(rec->cs, &rec->timestamp, rec->buffer);
+
+		(void)qb_thread_unlock(logt_wthread_lock);
 		free(rec->buffer);
 		free(rec);
 	}
@@ -113,7 +110,7 @@ qb_log_thread_priority_set(int32_t policy, int32_t priority)
 {
 	int res = 0;
 
-#if defined(HAVE_PTHREAD_SETSCHEDPARAM) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM)
 
 	logt_sched_policy = policy;
 
@@ -129,13 +126,13 @@ qb_log_thread_priority_set(int32_t policy, int32_t priority)
 	} else {
 		logt_sched_param.sched_priority = priority;
 	}
-	if (wthread_active == 0) {
+	if (wthread_active == QB_FALSE) {
 		logt_sched_param_queued = QB_TRUE;
 	} else {
 		res = pthread_setschedparam(logt_thread_id, policy,
 					    &logt_sched_param);
 		if (res != 0) {
-			res = -errno;
+			res = -res;
 		}
 	}
 #endif
@@ -146,50 +143,80 @@ int32_t
 qb_log_thread_start(void)
 {
 	int res;
+	qb_thread_lock_t *wthread_lock;
 
 	if (wthread_active) {
 		return 0;
 	}
 
-	wthread_active = 1;
+	wthread_active = QB_TRUE;
 	sem_init(&logt_thread_start, 0, 0);
 	sem_init(&logt_print_finished, 0, 0);
+	errno = 0;
+	logt_wthread_lock = qb_thread_lock_create(QB_THREAD_LOCK_SHORT);
+	if (logt_wthread_lock == NULL) {
+		return errno ? -errno : -1;
+	}
 	res = pthread_create(&logt_thread_id, NULL,
 			     qb_logt_worker_thread, NULL);
 	if (res != 0) {
-		wthread_active = 0;
-		return -errno;
+		wthread_active = QB_FALSE;
+		(void)qb_thread_lock_destroy(logt_wthread_lock);
+		return -res;
 	}
 	sem_wait(&logt_thread_start);
 
 	if (logt_sched_param_queued) {
 		res = qb_log_thread_priority_set(logt_sched_policy,
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM)
 		                                 logt_sched_param.sched_priority);
+#else
+		                                 0);
+#endif
 		if (res != 0) {
 			goto cleanup_pthread;
 		}
 		logt_sched_param_queued = QB_FALSE;
 	}
-	logt_wthread_lock = qb_thread_lock_create(QB_THREAD_LOCK_SHORT);
-	if (logt_wthread_lock == NULL) {
-		goto cleanup_pthread;
-	}
 
 	return 0;
 
 cleanup_pthread:
-	wthread_should_exit = 1;
+	wthread_should_exit = QB_TRUE;
 	sem_post(&logt_print_finished);
 	pthread_join(logt_thread_id, NULL);
+
+	wthread_active = QB_FALSE;
+	wthread_lock = logt_wthread_lock;
+	logt_wthread_lock = NULL;
+	(void)qb_thread_lock_destroy(wthread_lock);
+
 	sem_destroy(&logt_print_finished);
 	sem_destroy(&logt_thread_start);
 
 	return res;
 }
 
+
+void
+qb_log_thread_pause(struct qb_log_target *t)
+{
+	if (t->threaded) {
+		(void)qb_thread_lock(logt_wthread_lock);
+	}
+}
+
+void
+qb_log_thread_resume(struct qb_log_target *t)
+{
+	if (t->threaded) {
+		(void)qb_thread_unlock(logt_wthread_lock);
+	}
+}
+
 void
 qb_log_thread_log_post(struct qb_log_callsite *cs,
-		       time_t timestamp, const char *buffer)
+		       struct timespec *timestamp, const char *buffer)
 {
 	struct qb_log_record *rec;
 	size_t buf_size;
@@ -210,7 +237,7 @@ qb_log_thread_log_post(struct qb_log_callsite *cs,
 	}
 	memcpy(rec->buffer, buffer, buf_size);
 
-	rec->timestamp = timestamp;
+	memcpy(&rec->timestamp, timestamp, sizeof(struct timespec));
 
 	qb_list_init(&rec->list);
 	(void)qb_thread_lock(logt_wthread_lock);
@@ -242,34 +269,34 @@ qb_log_thread_stop(void)
 	int value;
 	struct qb_log_record *rec;
 
-	if (wthread_active == 0 && logt_wthread_lock == NULL) {
+	if (wthread_active == QB_FALSE && logt_wthread_lock == NULL) {
 		return;
 	}
-	if (wthread_active == 0) {
+	if (wthread_active == QB_FALSE) {
 		for (;;) {
 			res = sem_getvalue(&logt_print_finished, &value);
 			if (res != 0 || value == 0) {
-				return;
+				break;
 			}
 			sem_wait(&logt_print_finished);
 
 			(void)qb_thread_lock(logt_wthread_lock);
 
-			rec = qb_list_entry(logt_print_finished_records.next,
+			rec = qb_list_first_entry(&logt_print_finished_records,
 					    struct qb_log_record, list);
 			qb_list_del(&rec->list);
 			logt_memory_used = logt_memory_used -
 					   strlen(rec->buffer) -
 					   sizeof(struct qb_log_record) - 1;
-			(void)qb_thread_unlock(logt_wthread_lock);
 
-			qb_log_thread_log_write(rec->cs, rec->timestamp,
+			qb_log_thread_log_write(rec->cs, &rec->timestamp,
 						rec->buffer);
+			(void)qb_thread_unlock(logt_wthread_lock);
 			free(rec->buffer);
 			free(rec);
 		}
 	} else {
-		wthread_should_exit = 1;
+		wthread_should_exit = QB_TRUE;
 		sem_post(&logt_print_finished);
 		pthread_join(logt_thread_id, NULL);
 	}
