@@ -138,7 +138,9 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	void *shm_addr;
 	long page_size = sysconf(_SC_PAGESIZE);
 
-#ifdef QB_FORCE_SHM_ALIGN
+#ifdef QB_ARCH_HPPA
+	page_size = QB_MAX(page_size, 0x00400000); /* align to page colour */
+#elif defined(QB_FORCE_SHM_ALIGN)
 	page_size = QB_MAX(page_size, 16 * 1024);
 #endif /* QB_FORCE_SHM_ALIGN */
 	/* The user of this api expects the 'size' parameter passed into this function
@@ -153,7 +155,7 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	    sizeof(struct qb_ringbuffer_shared_s) + shared_user_data_size;
 
 	if (flags & QB_RB_FLAG_CREATE) {
-		file_flags |= O_CREAT | O_TRUNC;
+		file_flags |= O_CREAT | O_TRUNC | O_EXCL;
 	}
 
 	rb = calloc(1, sizeof(struct qb_ringbuffer_s));
@@ -164,7 +166,7 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	/*
 	 * Create a shared_hdr memory segment for the header.
 	 */
-	snprintf(filename, PATH_MAX, "qb-%s-header", name);
+	snprintf(filename, PATH_MAX, "%s-header", name);
 	fd_hdr = qb_sys_mmap_file_open(path, filename,
 				       shared_size, file_flags);
 	if (fd_hdr < 0) {
@@ -215,7 +217,7 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	 * They have to be separate.
 	 */
 	if (flags & QB_RB_FLAG_CREATE) {
-		snprintf(filename, PATH_MAX, "qb-%s-data", name);
+		snprintf(filename, PATH_MAX, "%s-data", name);
 		fd_data = qb_sys_mmap_file_open(path,
 						filename,
 						real_size, file_flags);
@@ -232,7 +234,7 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	}
 
 	qb_util_log(LOG_DEBUG,
-		    "shm size:%zd; real_size:%zd; rb->word_size:%d", size,
+		    "shm size:%ld; real_size:%ld; rb->word_size:%d", size,
 		    real_size, rb->shared_hdr->word_size);
 
 	/* this function closes fd_data */
@@ -288,22 +290,7 @@ qb_rb_close(struct qb_ringbuffer_s * rb)
 	qb_enter();
 
 	(void)qb_atomic_int_dec_and_test(&rb->shared_hdr->ref_count);
-	if (rb->flags & QB_RB_FLAG_CREATE) {
-		if (rb->notifier.destroy_fn) {
-			(void)rb->notifier.destroy_fn(rb->notifier.instance);
-		}
-		unlink(rb->shared_hdr->data_path);
-		unlink(rb->shared_hdr->hdr_path);
-		qb_util_log(LOG_DEBUG,
-			    "Free'ing ringbuffer: %s",
-			    rb->shared_hdr->hdr_path);
-	} else {
-		qb_util_log(LOG_DEBUG,
-			    "Closing ringbuffer: %s", rb->shared_hdr->hdr_path);
-	}
-	munmap(rb->shared_data, (rb->shared_hdr->word_size * sizeof(uint32_t)) << 1);
-	munmap(rb->shared_hdr, sizeof(struct qb_ringbuffer_shared_s));
-	free(rb);
+	(void)qb_rb_close_helper(rb, rb->flags & QB_RB_FLAG_CREATE, QB_FALSE);
 }
 
 void
@@ -314,24 +301,8 @@ qb_rb_force_close(struct qb_ringbuffer_s * rb)
 	}
 	qb_enter();
 
-	if (rb->notifier.destroy_fn) {
-		(void)rb->notifier.destroy_fn(rb->notifier.instance);
-	}
-
-        errno = 0;
-	unlink(rb->shared_hdr->data_path);
-	qb_util_perror(LOG_DEBUG,
-		    "Force free'ing ringbuffer: %s",
-		    rb->shared_hdr->data_path);
-
-        errno = 0;
-	unlink(rb->shared_hdr->hdr_path);
-	qb_util_perror(LOG_DEBUG,
-		    "Force free'ing ringbuffer: %s",
-		    rb->shared_hdr->hdr_path);
-	munmap(rb->shared_data, (rb->shared_hdr->word_size * sizeof(uint32_t)) << 1);
-	munmap(rb->shared_hdr, sizeof(struct qb_ringbuffer_shared_s));
-	free(rb);
+	qb_atomic_int_set(&rb->shared_hdr->ref_count, -1);
+	(void)qb_rb_close_helper(rb, QB_TRUE, QB_TRUE);
 }
 
 char *
@@ -451,8 +422,7 @@ qb_rb_chunk_alloc(struct qb_ringbuffer_s * rb, size_t len)
 		while (qb_rb_space_free(rb) < (len + QB_RB_CHUNK_MARGIN)) {
 			int rc = _rb_chunk_reclaim(rb);
 			if (rc != 0) {
-				errno = rc;
-				return NULL;
+				return NULL;  /* errno already set */
 			}
 		}
 	} else {
@@ -571,7 +541,8 @@ _rb_chunk_reclaim(struct qb_ringbuffer_s * rb)
 	old_read_pt = rb->shared_hdr->read_pt;
 	chunk_magic = QB_RB_CHUNK_MAGIC_GET(rb, old_read_pt);
 	if (chunk_magic != QB_RB_CHUNK_MAGIC) {
-		return -EINVAL;
+		errno = EINVAL;
+		return -errno;
 	}
 
 	old_chunk_size = QB_RB_CHUNK_SIZE_GET(rb, old_read_pt);
@@ -648,7 +619,11 @@ qb_rb_chunk_peek(struct qb_ringbuffer_s * rb, void **data_out, int32_t timeout)
 		if (rb->notifier.post_fn) {
 			(void)rb->notifier.post_fn(rb->notifier.instance, res);
 		}
-		return 0;
+#ifdef EBADMSG
+		return -EBADMSG;
+#else
+		return -EINVAL;
+#endif
 	}
 	chunk_size = QB_RB_CHUNK_SIZE_GET(rb, read_pt);
 	*data_out = QB_RB_CHUNK_DATA_GET(rb, read_pt);
@@ -723,12 +698,12 @@ print_header(struct qb_ringbuffer_s * rb)
 	} else {
 		printf(" ->NORMAL\n");
 	}
-	printf(" ->write_pt [%d]\n", rb->shared_hdr->write_pt);
-	printf(" ->read_pt [%d]\n", rb->shared_hdr->read_pt);
-	printf(" ->size [%d words]\n", rb->shared_hdr->word_size);
 #ifndef S_SPLINT_S
-	printf(" =>free [%zu bytes]\n", qb_rb_space_free(rb));
-	printf(" =>used [%zu bytes]\n", qb_rb_space_used(rb));
+	printf(" ->write_pt [%" PRIu32 "]\n", rb->shared_hdr->write_pt);
+	printf(" ->read_pt [%" PRIu32 "]\n", rb->shared_hdr->read_pt);
+	printf(" ->size [%" PRIu32 " words]\n", rb->shared_hdr->word_size);
+	printf(" =>free [%zd bytes]\n", qb_rb_space_free(rb));
+	printf(" =>used [%zd bytes]\n", qb_rb_space_used(rb));
 #endif /* S_SPLINT_S */
 }
 
@@ -886,7 +861,11 @@ qb_rb_create_from_file(int32_t fd, uint32_t flags)
 	 * 6. data
 	 */
 	n_required = (word_size * sizeof(uint32_t));
-	rb = qb_rb_open("create_from_file", n_required,
+
+	/*
+	 * qb_rb_open adds QB_RB_CHUNK_MARGIN + 1 to the requested size.
+	 */
+	rb = qb_rb_open("create_from_file", n_required - (QB_RB_CHUNK_MARGIN + 1),
 			QB_RB_FLAG_CREATE | QB_RB_FLAG_NO_SEMAPHORE, 0);
 	if (rb == NULL) {
 		return NULL;
